@@ -1,5 +1,9 @@
 #!/usr/bin/env python3
-import pkg_resources
+from __future__ import annotations
+
+from importlib.metadata import version as get_version
+
+from packaging.version import parse
 
 extensions = [
     'sphinx.ext.autodoc',
@@ -14,7 +18,7 @@ project = 'smtpproto'
 author = 'Alex Grönholm'
 copyright = '2020, ' + author
 
-v = pkg_resources.get_distribution(project).parsed_version
+v = parse(get_version('smtpproto'))
 version = v.base_version
 release = v.public
 
