@@ -1,5 +1,7 @@
+from __future__ import annotations
+
 from abc import ABCMeta, abstractmethod
-from base64 import b64encode, b64decode
+from base64 import b64decode, b64encode
 from dataclasses import dataclass
 from typing import AsyncGenerator
 
@@ -86,7 +88,7 @@ class OAuth2Authenticator(SMTPAuthenticator):
     """
 
     def __init__(self, username: str):
-        self.username = username
+        self.username: str = username
 
     @property
     def mechanism(self) -> str:
