@@ -1,12 +1,14 @@
+from __future__ import annotations
+
 import re
 from dataclasses import dataclass, field
 from email.headerregistry import Address
 from email.message import EmailMessage
-from email.policy import SMTPUTF8, SMTP, Policy
+from email.policy import SMTP, SMTPUTF8, Policy
 from enum import Enum, auto
-from typing import Iterable, Optional, List, Union, Tuple, FrozenSet, Sequence, NoReturn
+from typing import Iterable, NoReturn, Pattern, Sequence
 
-response_re = re.compile('(\\d+)([- ])(.*)$')
+response_re: Pattern[str] = re.compile('(\\d+)([- ])(.*)$')
 
 
 class ClientState(Enum):
@@ -59,15 +61,16 @@ class SMTPClientProtocol:
     """The (E)SMTP protocol state machine."""
 
     _state: ClientState = field(init=False, default=ClientState.greeting_expected)
+    _smtputf8_message: bool = field(init=False, default=True)
     _out_buffer: bytes = field(init=False, default=b'')
     _in_buffer: bytes = field(init=False, default=b'')
-    _response_code: Optional[int] = field(init=False, default=None)
-    _response_lines: List[str] = field(init=False, default_factory=list)
-    _command_sent: Optional[str] = field(init=False, default=None)
-    _args_sent: Optional[Tuple[bytes, ...]] = field(init=False, default=None)
-    _extensions: FrozenSet[str] = field(init=False, default_factory=frozenset)
-    _auth_mechanisms: FrozenSet[str] = field(init=False, default_factory=frozenset)
-    _max_message_size: Optional[int] = field(init=False, default=None)
+    _response_code: int | None = field(init=False, default=None)
+    _response_lines: list[str] = field(init=False, default_factory=list)
+    _command_sent: str | None = field(init=False, default=None)
+    _args_sent: tuple[bytes, ...] | None = field(init=False, default=None)
+    _extensions: frozenset[str] = field(init=False, default_factory=frozenset)
+    _auth_mechanisms: frozenset[str] = field(init=False, default_factory=frozenset)
+    _max_message_size: int | None = field(init=False, default=None)
 
     def _require_state(self, *states: ClientState) -> None:
         if self._state not in states:
@@ -85,24 +88,31 @@ class SMTPClientProtocol:
             raise SMTPUnsupportedAuthMechanism(
                 f'{mechanism} is not a supported authentication mechanism on this server')
 
-    def _encode_address(self, address: Union[str, Address]) -> bytes:
+    def _encode_address(self, address: str | Address) -> bytes:
         if isinstance(address, Address):
             address_str = f'{address.username}@{address.domain}'
         else:
             address_str = address
 
-        if 'SMTPUTF8' in self._extensions:
+        if self._smtputf8_message:
             return address_str.encode('utf-8')
 
         # If SMPTUTF8 is not supported, the address must be ASCII compatible
         try:
             return address_str.encode('ascii')
         except UnicodeEncodeError:
-            raise SMTPProtocolViolation(
-                f'The address {address_str!r} requires UTF-8 encoding but the server does not '
-                f'support the SMTPUTF8 extension')
+            if 'SMTPUTF8' in self._extensions:
+                raise SMTPProtocolViolation(
+                    f'The address {address_str!r} requires UTF-8 encoding but `smtputf8` was not '
+                    'specified in the mail command'
+                )
+            else:
+                raise SMTPProtocolViolation(
+                    f'The address {address_str!r} requires UTF-8 encoding but the server does not '
+                    'support the SMTPUTF8 extension'
+                )
 
-    def _send_command(self, command: str, *args: Union[str, bytes]) -> None:
+    def _send_command(self, command: str, *args: str | bytes) -> None:
         if self._command_sent is not None:
             raise SMTPProtocolViolation('Tried to send a command before the previous one received '
                                         'a response')
@@ -117,7 +127,7 @@ class SMTPClientProtocol:
         self._args_sent = args_encoded
 
     def _parse_extensions(self, lines: Iterable[str]) -> None:
-        auth_mechanisms: List[str] = []
+        auth_mechanisms: list[str] = []
         extensions = []
         for line in lines:
             extension, *params = line.split(' ')
@@ -133,7 +143,7 @@ class SMTPClientProtocol:
         self._extensions = frozenset(extensions)
         self._auth_mechanisms = frozenset(auth_mechanisms)
 
-    def _parse_response(self, code: int, lines: Sequence[str]) -> Optional[SMTPResponse]:
+    def _parse_response(self, code: int, lines: Sequence[str]) -> SMTPResponse | None:
         command, args = self._command_sent, self._args_sent or ()
         self._command_sent = self._args_sent = None
 
@@ -236,21 +246,21 @@ class SMTPClientProtocol:
         return buffer
 
     @property
-    def max_message_size(self) -> Optional[int]:
+    def max_message_size(self) -> int | None:
         """The maximum size of the email message (in bytes) accepted by the server."""
         return self._max_message_size
 
     @property
-    def auth_mechanisms(self) -> FrozenSet[str]:
+    def auth_mechanisms(self) -> frozenset[str]:
         """The set of authentication mechanisms supported on the server."""
         return self._auth_mechanisms
 
     @property
-    def extensions(self) -> FrozenSet[str]:
+    def extensions(self) -> frozenset[str]:
         """The set of extensions advertised by the server."""
         return self._extensions
 
-    def authenticate(self, mechanism: str, secret: Optional[str] = None) -> None:
+    def authenticate(self, mechanism: str, secret: str | None = None) -> None:
         """
         Authenticate to the server using the given mechanism and an accompanying secret.
 
@@ -297,11 +307,12 @@ class SMTPClientProtocol:
         """Send the QUIT command (required to cleanly shut down the session)."""
         self._send_command('QUIT')
 
-    def mail(self, sender: Union[str, Address]) -> None:
+    def mail(self, sender: str | Address, *, smtputf8: bool = True) -> None:
         """
         Send the MAIL FROM command (starts a mail transaction).
 
         :param sender: the sender's email address
+        :param smtputf8: send the SMTPUTF8 option, if available on the server
 
         """
         self._require_state(ClientState.ready, ClientState.authenticated)
@@ -309,12 +320,16 @@ class SMTPClientProtocol:
         args = []
         if '8BITMIME' in self._extensions:
             args.append('BODY=8BITMIME')
-        if 'SMTPUTF8' in self._extensions:
+
+        if smtputf8 and 'SMTPUTF8' in self._extensions:
+            self._smtputf8_message = True
             args.append('SMTPUTF8')
+        else:
+            self._smtputf8_message = False
 
         self._send_command('MAIL', b'FROM:<' + self._encode_address(sender) + b'>', *args)
 
-    def recipient(self, recipient: Union[str, Address]) -> None:
+    def recipient(self, recipient: str | Address) -> None:
         """
         Send the RCPT TO command (declare an intended recipient).
 
@@ -346,7 +361,7 @@ class SMTPClientProtocol:
 
         """
         self._require_state(ClientState.send_data)
-        policy: Policy = SMTPUTF8 if 'SMTPUTF8' in self._extensions else SMTP
+        policy: Policy = SMTPUTF8 if self._smtputf8_message else SMTP
         policy = policy.clone(cte_type='7bit') if '8BITMIME' not in self._extensions else policy
         self._out_buffer += message.as_bytes(policy=policy).replace(b'\r\n.', b'\r\n..')
         self._out_buffer += b'.\r\n'
@@ -363,7 +378,7 @@ class SMTPClientProtocol:
         self._require_extension('STARTTLS')
         self._send_command('STARTTLS')
 
-    def feed_bytes(self, data: bytes) -> Optional[SMTPResponse]:
+    def feed_bytes(self, data: bytes) -> SMTPResponse | None:
         """
         Feed received bytes from the transport into the state machine.
 
