@@ -1,13 +1,15 @@
+from __future__ import annotations
+
 import ssl
-from contextlib import contextmanager, ExitStack
+from contextlib import ExitStack, closing, contextmanager
 from email.headerregistry import Address
 from email.message import EmailMessage
-from typing import Optional
+from socket import socket
 
 import pytest
 from aiosmtpd.controller import Controller
 from aiosmtpd.handlers import Sink
-from aiosmtpd.smtp import SMTP, syntax
+from aiosmtpd.smtp import SMTP, AuthResult
 
 from smtpproto.auth import PlainAuthenticator
 from smtpproto.client import AsyncSMTPClient, SyncSMTPClient
@@ -18,24 +20,26 @@ pytestmark = pytest.mark.anyio
 
 class DummyController(Controller):
     def __init__(self, handler, factory=SMTP, hostname=None, port=0, *, ready_timeout=1.0,
-                 enable_SMTPUTF8=True, ssl_context=None):
+                 ssl_context=None):
         super().__init__(handler, hostname=hostname, port=port, ready_timeout=ready_timeout,
-                         enable_SMTPUTF8=enable_SMTPUTF8, ssl_context=None)
+                         ssl_context=None)
         self.__factory = factory
         self.__ssl_context = ssl_context
 
     def factory(self):
-        return self.__factory(self.handler, enable_SMTPUTF8=self.enable_SMTPUTF8,
-                              hostname=self.hostname, tls_context=self.__ssl_context)
+        return self.__factory(self.handler, hostname=self.hostname, tls_context=self.__ssl_context)
 
 
 @contextmanager
-def start_server(*, ssl_context: Optional[ssl.SSLContext] = None, factory=SMTP,
+def start_server(*, ssl_context: ssl.SSLContext | None = None, factory=SMTP,
                  handler: type = Sink):
-    controller = DummyController(handler, factory=factory, hostname='localhost',
+    with closing(socket()) as sock:
+        sock.bind(('localhost', 0))
+        port = sock.getsockname()[1]
+
+    controller = DummyController(handler, factory=factory, hostname='localhost', port=port,
                                  ssl_context=ssl_context)
     controller.start()
-    port = controller.server.sockets[0].getsockname()[1]
     yield controller.hostname, port
     controller.stop()
 
@@ -62,32 +66,28 @@ class TestAsyncClient:
             async with AsyncSMTPClient(host=host, port=port):
                 pass
 
-    @pytest.mark.parametrize('use_tls', [False, True], ids=['notls', 'tls'])
     @pytest.mark.parametrize('success', [True, False], ids=['success', 'failure'])
-    async def test_auth_plain(self, client_context, server_context, use_tls, success):
-        class AuthCapableHandler:
-            @staticmethod
-            async def handle_EHLO(server, session, envelope, hostname):
-                await server.push('250-AUTH PLAIN')
-                return '250 HELP'
-
+    async def test_auth_plain(self, client_context, server_context, success):
         class AuthCapableSMTP(SMTP):
-            @syntax('AUTH <secret>')
-            async def smtp_AUTH(self, arg):
-                credentials = arg.split(' ')[1]
+            async def auth_PLAIN(self, _, args: list[str]) -> AuthResult:
                 expected = 'AHVzZXJuYW1lAHBhc3N3b3Jk'
-                if credentials == expected and success:
-                    await self.push('235 Authentication successful')
+                if args[1] == expected and success:
+                    return AuthResult(success=True)
                 else:
-                    await self.push('535 Invalid credentials')
+                    return AuthResult(success=False, handled=False)
+
+        stack = ExitStack()
+        host, port = stack.enter_context(
+            start_server(ssl_context=server_context, factory=AuthCapableSMTP)
+        )
+        if not success:
+            stack.enter_context(pytest.raises(SMTPException))
 
         authenticator = PlainAuthenticator('username', 'password')
-        with start_server(ssl_context=server_context if use_tls else None, factory=AuthCapableSMTP,
-                          handler=AuthCapableHandler) as (host, port):
-            with ExitStack() if success else pytest.raises(SMTPException):
-                async with AsyncSMTPClient(host=host, port=port, ssl_context=client_context,
-                                           authenticator=authenticator):
-                    pass
+        with stack:
+            async with AsyncSMTPClient(host=host, port=port, ssl_context=client_context,
+                                       authenticator=authenticator):
+                pass
 
 
 class TestSyncClient:
@@ -112,29 +112,24 @@ class TestSyncClient:
             with SyncSMTPClient(host=host, port=port):
                 pass
 
-    @pytest.mark.parametrize('use_tls', [False, True], ids=['notls', 'tls'])
     @pytest.mark.parametrize('success', [True, False], ids=['success', 'failure'])
-    def test_auth_plain(self, client_context, server_context, use_tls, success):
-        class AuthCapableHandler:
-            @staticmethod
-            async def handle_EHLO(server, session, envelope, hostname):
-                await server.push('250-AUTH PLAIN')
-                return '250 HELP'
-
+    def test_auth_plain(self, client_context, server_context, success):
         class AuthCapableSMTP(SMTP):
-            @syntax('AUTH <secret>')
-            async def smtp_AUTH(self, arg):
-                credentials = arg.split(' ')[1]
+            async def auth_PLAIN(self, _, args: list[str]) -> AuthResult:
                 expected = 'AHVzZXJuYW1lAHBhc3N3b3Jk'
-                if credentials == expected and success:
-                    await self.push('235 Authentication successful')
+                if args[1] == expected and success:
+                    return AuthResult(success=True)
                 else:
-                    await self.push('535 Invalid credentials')
+                    return AuthResult(success=False, handled=False)
+
+        stack = ExitStack()
+        host, port = stack.enter_context(
+            start_server(ssl_context=server_context, factory=AuthCapableSMTP)
+        )
+        if not success:
+            stack.enter_context(pytest.raises(SMTPException))
 
         authenticator = PlainAuthenticator('username', 'password')
-        with start_server(ssl_context=server_context if use_tls else None, factory=AuthCapableSMTP,
-                          handler=AuthCapableHandler) as (host, port):
-            with ExitStack() if success else pytest.raises(SMTPException):
-                with SyncSMTPClient(host=host, port=port, ssl_context=client_context,
-                                    authenticator=authenticator):
-                    pass
+        with stack, SyncSMTPClient(host=host, port=port, ssl_context=client_context,
+                                   authenticator=authenticator):
+            pass
