@@ -1,11 +1,13 @@
+from __future__ import annotations
+
 from email.headerregistry import Address
 from email.message import EmailMessage
-from typing import Optional, Callable
+from typing import Callable
 
 import pytest
 
 from smtpproto.protocol import (
-    SMTPClientProtocol, ClientState, SMTPProtocolViolation, SMTPMissingExtension,
+    ClientState, SMTPClientProtocol, SMTPMissingExtension, SMTPProtocolViolation,
     SMTPUnsupportedAuthMechanism)
 
 
@@ -16,12 +18,13 @@ def call_protocol_method(protocol: SMTPClientProtocol, func: Callable,
     assert protocol.get_outgoing_data() == expected_outgoing_data
 
 
-def feed_bytes(protocol: SMTPClientProtocol, data: bytes, expected_code: Optional[int] = None,
-               expected_message: Optional[str] = None,
-               expected_state: Optional[ClientState] = None):
+def feed_bytes(protocol: SMTPClientProtocol, data: bytes, expected_code: int | None = None,
+               expected_message: str | None = None,
+               expected_state: ClientState | None = None):
     assert protocol.needs_incoming_data
     response = protocol.feed_bytes(data)
     if expected_code:
+        assert response
         assert response.code == expected_code
         assert not protocol.needs_incoming_data
     else:
@@ -29,6 +32,7 @@ def feed_bytes(protocol: SMTPClientProtocol, data: bytes, expected_code: Optiona
         assert protocol.needs_incoming_data
 
     if expected_message:
+        assert response
         assert response.message == expected_message
     if expected_state:
         assert protocol.state is expected_state
@@ -63,15 +67,15 @@ def exchange_greetings(protocol, esmtp=True):
         assert protocol.max_message_size is None
 
 
-def start_mail_tx(protocol):
+def start_mail_tx(protocol, smtputf8=True):
     # Start a mail transaction
     extra_args = b''
     if '8BITMIME' in protocol.extensions:
         extra_args += b' BODY=8BITMIME'
-    if 'SMTPUTF8' in protocol.extensions:
+    if smtputf8 and 'SMTPUTF8' in protocol.extensions:
         extra_args += b' SMTPUTF8'
 
-    call_protocol_method(protocol, lambda: protocol.mail('foo@bar.com'),
+    call_protocol_method(protocol, lambda: protocol.mail('foo@bar.com', smtputf8=smtputf8),
                          b'MAIL FROM:<foo@bar.com>' + extra_args + b'\r\n')
     feed_bytes(protocol, b'250 OK\r\n', 250, 'OK', ClientState.mailtx)
 
@@ -108,14 +112,17 @@ def protocol():
     return proto
 
 
-@pytest.mark.parametrize('esmtp, expected_cte, expected_subject, expected_body', [
-    pytest.param(True, '8bit', 'This is a subjëct', 'This is ä test message.', id='8bit'),
-    pytest.param(False, 'base64', 'This is a =?utf-8?q?subj=C3=ABct?=',
-                 'VGhpcyBpcyDDpCB0ZXN0IG1lc3NhZ2UuCg==', id='7bit')
+@pytest.mark.parametrize('esmtp, smtputf8, expected_cte, expected_subject, expected_body', [
+    pytest.param(True, True, '8bit', 'This is a subjëct', 'This is ä test message.', id='8bit'),
+    pytest.param(False, True, 'base64', 'This is a =?utf-8?q?subj=C3=ABct?=',
+                 'VGhpcyBpcyDDpCB0ZXN0IG1lc3NhZ2UuCg==', id='7bit'),
+    pytest.param(True, False, '8bit', 'This is a =?utf-8?q?subj=C3=ABct?=',
+                 'This is ä test message.', id='smtputf8_opt_out')
 ])
-def test_send_mail_utf8_content(protocol, esmtp, expected_cte, expected_subject, expected_body):
+def test_send_mail_utf8_content(protocol, esmtp, smtputf8, expected_cte, expected_subject,
+                                expected_body):
     exchange_greetings(protocol, esmtp=esmtp)
-    start_mail_tx(protocol)
+    start_mail_tx(protocol, smtputf8=smtputf8)
 
     message = EmailMessage()
     message['Subject'] = 'This is a subjëct'
@@ -125,7 +132,7 @@ def test_send_mail_utf8_content(protocol, esmtp, expected_cte, expected_subject,
                          f'Content-Type: text/plain; charset="utf-8"\r\n'
                          f'Content-Transfer-Encoding: {expected_cte}\r\n'
                          f'MIME-Version: 1.0\r\n\r\n'
-                         f'{expected_body}\r\n.\r\n'.encode('utf-8'))
+                         f'{expected_body}\r\n.\r\n'.encode())
     feed_bytes(protocol, b'250 OK\r\n', 250, 'OK', ClientState.ready)
 
 
@@ -140,7 +147,19 @@ def test_send_mail_utf8_addresses(protocol, unicode_address):
 def test_send_mail_unicode_sender_encoding_error(protocol, unicode_address):
     exchange_greetings(protocol, esmtp=False)
     exc = pytest.raises(SMTPProtocolViolation, protocol.mail, unicode_address)
-    exc.match("^The address 'héllö@example.org' requires UTF-8")
+    exc.match(
+        "^The address 'héllö@example.org' requires UTF-8 encoding but the server does not support "
+        "the SMTPUTF8 extension"
+    )
+
+
+def test_send_mail_unicode_sender_no_smtputf8_encoding_error(protocol, unicode_address):
+    exchange_greetings(protocol, esmtp=True)
+    exc = pytest.raises(SMTPProtocolViolation, protocol.mail, unicode_address,
+                        smtputf8=False)
+    exc.match(
+        "^The address 'héllö@example.org' requires UTF-8 encoding but `smtputf8` was not specified"
+    )
 
 
 def test_send_mail_unicode_recipient_encoding_error(protocol, unicode_address):
@@ -152,6 +171,15 @@ def test_send_mail_unicode_recipient_encoding_error(protocol, unicode_address):
     exc.match("^The address 'héllö@example.org' requires UTF-8")
 
 
+def test_send_mail_unicode_recipient_no_smtputf8_encoding_error(protocol, unicode_address):
+    exchange_greetings(protocol, esmtp=True)
+    protocol.mail('hello@example.org', smtputf8=False)
+    feed_bytes(protocol, b'250 OK\r\n', 250, 'OK', ClientState.mailtx)
+
+    exc = pytest.raises(SMTPProtocolViolation, protocol.recipient, unicode_address)
+    exc.match("^The address 'héllö@example.org' requires UTF-8")
+
+
 def test_send_mail_escape_dots(protocol):
     exchange_greetings(protocol)
     start_mail_tx(protocol)
@@ -159,13 +187,13 @@ def test_send_mail_escape_dots(protocol):
     message = EmailMessage()
     message.set_content('The following lines might trip the protocol:\n.test\n.')
     call_protocol_method(protocol, lambda: protocol.data(message),
-                         'Content-Type: text/plain; charset="utf-8"\r\n'
-                         'Content-Transfer-Encoding: 7bit\r\n'
-                         'MIME-Version: 1.0\r\n\r\n'
-                         'The following lines might trip the protocol:\r\n'
-                         '..test\r\n'
-                         '..\r\n'
-                         '.\r\n'.encode('utf-8'))
+                         b'Content-Type: text/plain; charset="utf-8"\r\n'
+                         b'Content-Transfer-Encoding: 7bit\r\n'
+                         b'MIME-Version: 1.0\r\n\r\n'
+                         b'The following lines might trip the protocol:\r\n'
+                         b'..test\r\n'
+                         b'..\r\n'
+                         b'.\r\n')
     feed_bytes(protocol, b'250 OK\r\n', 250, 'OK', ClientState.ready)
 
 
